/*******************************************************************************
 * Copyright (c) 2011, Daniel Murphy
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without modification,
 * are permitted provided that the following conditions are met:
 * 	* Redistributions of source code must retain the above copyright notice,
 * 	  this list of conditions and the following disclaimer.
 * 	* Redistributions in binary form must reproduce the above copyright notice,
 * 	  this list of conditions and the following disclaimer in the documentation
 * 	  and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 ******************************************************************************/
/**
 * Created at 4:31:15 AM Jan 15, 2011
 */
package org.jbox2d.testbed.tests;

import java.util.Vector;

import jp.tonyu.jbox2d.BodyBuilder;

import org.jbox2d.collision.shapes.CircleShape;
import org.jbox2d.collision.shapes.PolygonShape;
import org.jbox2d.common.Vec2;
import org.jbox2d.dynamics.Body;
import org.jbox2d.dynamics.BodyDef;
import org.jbox2d.dynamics.BodyType;
import org.jbox2d.dynamics.contacts.Contact;
import org.jbox2d.testbed.framework.TestbedSettings;
import org.jbox2d.testbed.framework.TestbedTest;

import android.util.Log;

/**
 * @author Daniel Murphy
 */
public class SphereStack extends TestbedTest {

	int e_count = 10;
	Vector<Body> m_bodies = new Vector<Body>();
	CircleShape circle = new CircleShape();

	/**
	 * @see org.jbox2d.testbed.framework.TestbedTest#initTest(boolean)
	 */
	@Override
	public void initTest(boolean argDeserialized) {
		{
			/*BodyDef bd = new BodyDef();
			Body ground = getWorld().createBody(bd);
*/
			PolygonShape shape = new PolygonShape();
			shape.setAsEdge(new Vec2(-40.0f, 0.0f), new Vec2(40.0f, -8.0f));
			//ground.createFixture(shape, 0.0f);
			builder().shape(shape).build();
		}

		{
			circle.m_radius = 1.0f;

			for (int i = 0; i < e_count; ++i)
			{
				/*BodyDef bd = new BodyDef();
				bd.type = BodyType.DYNAMIC;
				bd.position.set(0.0f+i*0.1f, 4.0f + 3.0f * i);

				Body b = getWorld().createBody(bd);
				m_bodies.add (b);

				b.createFixture(circle, 1.0f);*/

				m_bodies.add(builder()
						.type(BodyType.DYNAMIC)
						.pos(0.0+i*0.1, 4.0 + 3.0 * i)
						.shape(circle)
						.density(1.0)
						.build()
				);

				//m_bodies[i].setLinearVelocity(new Vec2(0.0f, -100.0f));
			}
		}
	}

	private BodyBuilder builder() {
		return new BodyBuilder(getWorld());
	}

	/**
	 * @see org.jbox2d.testbed.framework.TestbedTest#getTestName()
	 */
	@Override
	public String getTestName() {
		return "Rolling Sphere";
	}

	@Override
	public void beginContact(Contact contact) {
		Log.d("contact", contact+"");
		super.beginContact(contact);
	}
	@Override
	public synchronized void step(TestbedSettings settings) {
		super.step(settings);
		Vector<Body> willDestroy = new Vector<Body>();
		for (Body b:m_bodies) {
			Vec2 pos = b.getPosition();
			//b.applyForce(new Vec2(10,0), pos);
			if (pos.x>10) {
				getWorld().destroyBody(b);
				willDestroy.add(b);
			}
		}
		if (Math.random()<0.04) {
			/*BodyDef bd = new BodyDef();
			bd.type = BodyType.DYNAMIC;
			bd.position.set(-20.0f+(float)Math.random()*40f, 10.0f);
			Body b=getWorld().createBody(bd);
			b.createFixture(circle, 1.0f);
			m_bodies.add(b);*/
			m_bodies.add(builder()
					.type(BodyType.DYNAMIC)
					.pos(-15.0f+(float)Math.random()*30f, 10.0f)
					.shape(circle)
					.density(1.0)
					.build()
			);
		}
		for (Body b:willDestroy) {
			m_bodies.remove(b);
		}
	}
}
